////////////////////////////////////////////////////////////////////////
// Class:       MonitorSer
// Module Type: analyzer
// File:        MonitorSer_module.cc
//
// Generated at Thu May  9 15:59:44 2013 by Nicola Rossi using artmod
// from cetpkgsupport v1_02_00.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "artdaq/DAQdata/Fragments.hh"
#include "ds50daq/DAQ/V1495Fragment.hh"
#include "ds50daq/DAQ/V172xFragment.hh"

#include "TRootCanvas.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TH1I.h"
#include "TF1.h"
#include "TFile.h"

#include <numeric>
#include <vector>
#include <functional>
#include <algorithm>
#include <iostream>
#include <math.h>

namespace ds50 {
  class MonitorSer : public art::EDAnalyzer {
    public:
      explicit MonitorSer(fhicl::ParameterSet const & p);
      virtual ~MonitorSer();

      void analyze(art::Event const & e) override;
      void beginRun(art::Run const &) override;
      void endRun(art::Run const &) override;

    private:
      std::string board_type_;
      float start_us_, stop_us_;
      bool laser_only_;
      std::vector<float> range_;
      int single_channel_;
      int prescale_;
      std::string hists_file_;

      std::unique_ptr<TCanvas> canvas_[3];
      std::unique_ptr<TH1I> hists_[40];
      std::unique_ptr<TH1F> occupancy_;
      std::unique_ptr<TF1> fitfun_;
  };
}

ds50::MonitorSer::MonitorSer(fhicl::ParameterSet const & ps): board_type_(ps.get<std::string> ("board_type", "V1720")), start_us_(ps.get<float> ("start_us")), stop_us_(ps.get<float> ("stop_us")), laser_only_(ps.get<bool> ("laser_only", false)), range_(ps.get<std::vector<float>> ("range", std::vector<float>(0))), single_channel_(ps.get<int> ("single_channel", -1)), prescale_(ps.get<int> ("prescale", 100)), hists_file_(ps.get<std::string> ("hists_file", "")) { }

ds50::MonitorSer::~MonitorSer() {}

void ds50::MonitorSer::analyze(art::Event const & e) {
  art::Handle<artdaq::Fragments> v1495;
  e.getByLabel ("daq", "V1495", v1495);
  if (!v1495.isValid ()) return;

  if(v1495->size() != 1) {
    mf::LogError ("MonitorSer") << v1495->size() << " fragment for V1495";
    return;
  }
  if (laser_only_ && V1495Fragment((*v1495)[0]).trigger_type () != V1495Fragment::laser) return;

  for (int ch = 0; single_channel_ < 0 && ch < 40; ch++) if(V1495Fragment((*v1495)[0]).channel_in_trigger(ch)) occupancy_->Fill(ch);
  if (single_channel_ < 0) occupancy_->Fill(41);

  art::Handle<artdaq::Fragments> v172x;
  e.getByLabel ("daq", board_type_, v172x);
  if (!v172x.isValid ())  {
    e.getByLabel ("unhuff" + board_type_, board_type_, v172x);
    if (!v172x.isValid ()) return;
  }

  bool refresh = false;

  for (size_t i = 0; i < v172x->size(); ++i) {
    const auto& frag((*v172x)[i]);
    V172xFragment b(frag);

    if (!frag.hasMetadata ())   continue;
    int pre_samples = frag.metadata<V172xFragment::metadata> ()->record_length * (100 - frag.metadata<V172xFragment::metadata> ()->post_trigger) / 100;
    int start = int(start_us_ * frag.metadata<V172xFragment::metadata> ()->sample_rate_MHz) + pre_samples;
    int stop = int(stop_us_ * frag.metadata<V172xFragment::metadata> ()->sample_rate_MHz) + pre_samples;

    size_t board_id = b.board_id ();
    for(int ch=0; ch<8; ++ch) {
      int lg = board_id * 8 + ch;
      if (single_channel_ >= 0 && lg != single_channel_) continue;

      if (!b.channel_present (ch) || lg >= 40) continue;

      double baseline = (std::accumulate (b.chDataBegin (ch) + (2 * start - stop), b.chDataBegin (ch) + start, 0.) + std::accumulate (b.chDataBegin (ch) + stop, b.chDataBegin (ch) + (2 * stop - start), 0.)) / 2;
      double sum = std::accumulate (b.chDataBegin (ch) + start, b.chDataBegin (ch) + stop, 0.) - baseline;
      hists_[lg] -> Fill(sum);
      if (!refresh) refresh = !(int(hists_[lg]->GetEntries ()) % prescale_);
    }
  }

  if (single_channel_ >= 0 && refresh) {
    canvas_[0]->Modified ();
    canvas_[0]->Update ();
  } else if (refresh) {
    for (int k = 0; k < 40; k++) canvas_[k / 20]->cd (k % 20 + 1)->Modified ();
    canvas_[2]->Modified ();
    for (int i = 0; i < 3; i++) canvas_[i]->Update ();

  }
}

void ds50::MonitorSer::beginRun(art::Run const &) {
  for (auto &x: canvas_) x = 0;
  for (auto &x: hists_) x = 0;
  occupancy_ = 0;

  if (single_channel_ >= 0) {
    canvas_[0] = std::unique_ptr<TCanvas>(new TCanvas(Form("ch%d",0)));
    ((TRootCanvas*)canvas_[0]->GetCanvasImp())->DontCallClose();
    canvas_[0] -> Update();
  } else {
    for (int i = 0; i < 2; i++) {
      canvas_[i] = std::unique_ptr<TCanvas>(new TCanvas(Form("ch%d",i)));
      ((TRootCanvas*)canvas_[i]->GetCanvasImp ())->DontCallClose ();
      canvas_[i] -> Divide(4, 5);
      canvas_[i] -> Update();
    }
    for (int k = 0; k < 40; k++) canvas_[k / 20]->cd (k % 20 + 1)->SetGridx ();

    canvas_[2] = std::unique_ptr<TCanvas>(new TCanvas("laser occupancy"));
    ((TRootCanvas*)canvas_[2]->GetCanvasImp ())->DontCallClose ();
    canvas_[2]->Update();
    canvas_[2]->cd ();
    occupancy_ = std::unique_ptr<TH1F>(new TH1F("occupancy","Channel Occupancy in Laser",42,-0.5,41.5));
    occupancy_->Draw ();
  }


  for (int k = 0; k < 40; k++) {
    if (single_channel_ >= 0) {
      if (k != single_channel_) continue;
    } else {
      int lg_canvas = k % 20 + 1;
      canvas_[k / 20] -> cd(4 * ((lg_canvas - 1) % 5) + int((4 * lg_canvas - 1) / 20) + 1)->SetLogy ();
    }
    if (range_.size ()) hists_[k] = std::unique_ptr<TH1I>(new TH1I(Form ("Ser%d", k), "Ser", 200, range_[0], range_[1]));
    else {
      hists_[k] = std::unique_ptr<TH1I>(new TH1I(Form ("Ser%d", k), "Ser", 200, -50, 50));
      hists_[k] -> SetBit(TH1::kCanRebin);
    }
    hists_[k] -> SetTitle(Form("Title %d", k));
    hists_[k] -> SetFillColor(3);

    hists_[k] -> Draw ();
  }
}

namespace {
  float g_ (float x, float m, float s) {
    static const float pi = 2 * acosf(0);
    return expf(-0.5 * (x - m) * (x - m) / s / s) / sqrtf(2 * pi) / s;
  }

  float p_ (float lambda, int n) {
    double v = expf(-lambda);
    for (int i = 1; i <= n; i++) v *= lambda / i;
//    if (n > 1) v *= 1.1;
    return v;
  }

  float e_ (float x, float t, float s) {
    return 0.5 * expf ((s * s - 2 * x * t) / 2 / t / t) * (1. + erff ((-s + x * t / s) / sqrtf (2) / t));
  }

  double f_ (double *y, double *p) {
    float x = -y[0];
    float lambda = p[0];
    float N = p[1];
    float mb = 0;
    float sb = p[2];
    float mspe = p[3];
    float sspe = p[4];
    float tau = p[5];
    float Pexp = p[6];

    for (int i = 0; i < 7; i++) if (p[i] <= 0) return -1000;
    if (sspe > mspe * 0.5 || tau > mspe / 2) return -1000;
//    if (x < -4 * sb) return 0;

    float baseline = p_ (lambda, 0) * g_ (x, mb, sb);
    float spe = 0;
    for (int i = 1; i < 10; i++) {
      float o = p_ (lambda, i);
      spe += o * g_ (x, mspe * i, sqrtf (sspe * sspe * i + sb * sb));
    }
    float expo = e_ (x, tau, sb);

    double r = N * (baseline + spe + Pexp * expo);
    return std::isnormal (r) ? r : -100;
  }

  float fitter (TH1& h, TF1& f) {
    if (h.GetMean () > -2) {
      f.SetParameters(0, 0, 0, 0, 0, 0, 0);
      return 0;
    }

    float integral = h.GetBinWidth (1) * h.GetEntries ();
    f.SetParameters(0.2, integral, 10, 80, std::min(h.GetRMS (), 20.), 20, 0.01);
    f.SetParLimits (0, 0, 10);   //  lambda
    f.SetParLimits (1, integral * 0.7, integral * 1.2);    //  N
    f.SetParLimits (2, 5, 15);   //  baseline
    f.SetParLimits (3, 20, 200); //  mu spe
    f.SetParLimits (5, 2, 100);  //  tau
//    fitfun_->FixParameter (5, 2);
//    fitfun_->FixParameter (6, 0);

    float chi2;
    for (int i = 0; i < 10; i++) {
      h.Fit(&f, "QBM");
      chi2 = f.GetChisquare () / f.GetNDF ();
//      std::cout << "fit test " << i << " chi2/NDF " << chi2 << std::endl;
      double l_,h_;
      f.GetParLimits (3, l_, h_);
      if (f.GetParameter (3) >= h_ * 0.9) {
//      f.SetParameter (3, 80);
        continue;
      }
      if (f.GetParameter (4) < f.GetParameter(3) / 5 || f.GetParameter(4) > f.GetParameter(3) * 0.4) {
        f.SetParameter (4, f.GetParameter(3) * 0.3);
        continue;
      }
      if (chi2 < 0.6 || chi2 > 5) continue;

      h.Draw ();
      break;
    }
    return chi2;
  }
}



void ds50::MonitorSer::endRun(art::Run const &) {
  fitfun_ = std::unique_ptr<TF1>(new TF1("fitfun", f_, -1000, 50, 7));

  if (single_channel_ < 0) {
    occupancy_->Scale (1. / occupancy_->GetBinContent (42));
    occupancy_->SetBinContent (42, 0);
    canvas_[2]->Modified ();
    canvas_[2]->Update ();
  }

  std::unique_ptr<TFile> f;
  if (hists_file_.size ()) {
    f = std::unique_ptr<TFile>(new TFile (hists_file_.c_str (), "recreate"));
    if (single_channel_ < 0) occupancy_->Write ();
  }


  std::cout << "FIT RESULTS: " << std::endl;
  std::cout << "ch chi2 lambda N sb mspe sspe tau Pexp" << std::endl;
  for (int k = 0; k < 40; k++) {
    if (single_channel_ >= 0) {
      if (k != single_channel_) continue;
    } else {
      int lg_canvas = k % 20 + 1;
      canvas_[k / 20] -> cd(4 * ((lg_canvas - 1) % 5) + int((4 * lg_canvas - 1) / 20) + 1);
    }

    if (f) hists_[k]->Write ();

    float chi2 = fitter (*hists_[k], *fitfun_);
    std::cout << k << " " << chi2 << " ";
    for(int i = 0; i < 7; i++) std::cout << fitfun_->GetParameter (i) << " ";
    std::cout << std::endl;

    if (single_channel_ >= 0) canvas_[0] -> Update();
    else canvas_[k / 20] -> Update();
  }
}

DEFINE_ART_MODULE(ds50::MonitorSer)
