////////////////////////////////////////////////////////////////////////
// Class:       WFCanvas
// Module Type: analyzer
// File:        WFCanvas_module.cc
//
// Generated at Sun Dec  2 12:23:06 2012 by Alessandro Razeto using artmod
// from art v1_02_04.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"
#include "art/Utilities/InputTag.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "artdaq/DAQdata/Fragments.hh"
#include "ds50daq/DAQ/V172xFragment.hh"

#include "TCanvas.h"
#include "TGraph.h"

#include <numeric>
#include <vector>

namespace ds50 {
  class WFCanvas : public art::EDAnalyzer {
    public:
      explicit WFCanvas (fhicl::ParameterSet const & p);
      virtual ~WFCanvas ();

      void analyze (art::Event const & e) override;
      void beginJob() override;

    private:
      art::InputTag raw_label_;
      std::unique_ptr<TCanvas> canvas_;
      std::unique_ptr<TGraph> graph_;
      std::vector<Float_t> x_;
      int prescale_;
      int channel_;
  };
}


ds50::WFCanvas::WFCanvas (fhicl::ParameterSet const & ps): raw_label_(ps.get<std::string> ("raw_label")), prescale_(ps.get<int> ("prescale")), channel_(ps.get<int> ("channel")) {
}

ds50::WFCanvas::~WFCanvas() { }

void ds50::WFCanvas::analyze (art::Event const & e) {
  art::Handle<artdaq::Fragments> raw;
  e.getByLabel (raw_label_, raw);

  for (size_t i = 0; i < raw->size(); ++i) {
    const auto& frag((*raw)[i]);
    if (frag.sequenceID () % prescale_) return;

    std::cout << frag.sequenceID () << std::endl;

    V172xFragment b(frag);
    size_t board_id = b.board_id ();
    if (board_id) continue;

    size_t n_samples = std::distance (b.chDataBegin (1), b.chDataEnd(1));
    if (!x_.size ()) {
      x_.resize (n_samples);
      std::iota (x_.begin (), x_.end (), 0);
      std::for_each (x_.begin(), x_.end (), [](Float_t& d) { d *= 4;});
    }
    std::vector<Float_t> y(n_samples);
    std::copy (b.chDataBegin (channel_), b.chDataEnd(channel_), y.begin ());

    canvas_->Clear ();
    graph_->DrawGraph (n_samples, &(x_[0]), &(y[0]), "AL");
    canvas_->Modified ();
    canvas_->Update ();
  }
}

void ds50::WFCanvas::beginJob() { 
  canvas_ = std::unique_ptr<TCanvas>(new TCanvas("wf"));
  graph_ = std::unique_ptr<TGraph>(new TGraph);
  graph_->SetTitle ("Waveform");

  canvas_->Update ();
}

DEFINE_ART_MODULE(ds50::WFCanvas)
