#!/bin/bash
#

# build_ds50daq.sh <product_dir> <e4|e4:eth> <debug|opt|prof>

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e4|e4:eth> <debug|opt|prof>"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    mydir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    qualdot=`echo ${basequal} | sed -e 's/:/./g'`.${extraqual}
    qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	option="-o"
    elif [ "${extraqual}" = "debug" ]
    then
	option="-d"
    elif [ "${extraqual}" = "prof" ]
    then
	option="-p"
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/cet

}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specifiy your product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=ds50daq
pkgver=v0_04_06
ssibuildshims_version=v0_06_06
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=${package}-v0_04_06

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
else
   rm -rf ${blddir}/*
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

set -x
cd ${sourcedir}
tar xf ${tardir}/${pkgtarfile}.tar
set +x

cd ${blddir}
source ${sourcedir}/${pkgtarfile}/ups/setup_for_development ${option} ${basequal}
set -x
cmake -DCMAKE_INSTALL_PREFIX=${product_dir} -DCMAKE_BUILD_TYPE=$CETPKG_TYPE $CETPKG_SOURCE
make -j4
make install
make package

cd ${mydir}
mv ${blddir}/${package}*.tar.bz2 .

set +x

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "ds50daq is installed at ${DS50DAQ_FQ_DIR}"

exit 0
